/*

    Zelda Time to Triumph

    Copyright (C) 2007-2009  Vincent Jouillat

    Please send bugreports with examples or suggestions to www.zeldaroth.fr

*/

#include <sstream>
#include <fstream>
#include <iostream>

#include <SDL2/SDL.h>

#include "Audio.h"

Audio::Audio() : volume(0), musiqueId(0), specialId(0) {
    SOUND = FSOUND_Init(44100, 32, 0);
    music = NULL;

    if (SOUND) {
        previous_volume = -1;
        previous_volson = FSOUND_GetSFXMasterVolume();
        loadSounds();
    }
}

Audio::~Audio() {
    if (SOUND) {
        freeSounds();
        FMUSIC_StopSong(music);
        FMUSIC_SetMasterVolume(music, previous_volume);
        FMUSIC_FreeSong(music);
        FSOUND_SetSFXMasterVolume(previous_volson);
        FSOUND_Close();
    }
}

void Audio::setVolume(int vol) {volume=vol*4;
    if (previous_volume == -1) previous_volume = FMUSIC_GetMasterVolume(music);
    FMUSIC_SetMasterVolume(music, volume);}
void Audio::setVolson(int volson) {FSOUND_SetSFXMasterVolume(volson*4);}

void Audio::loadSounds() {
    sons = new FSOUND_SAMPLE*[44];

    sons[0] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/text.ogg",0,0,0); // lettres
    sons[1] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/menu1.ogg",0,0,0); // menu 1
    sons[2] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/menu2.ogg",0,0,0); // menu 2
    sons[3] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/menu3.ogg",0,0,0); // menu 3
    sons[4] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/menu4.ogg",0,0,0); // menu 4
    sons[5] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/timewarp.ogg",0,0,0); // time retour
    sons[6] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/tombe.ogg",0,0,0); // tombe (ennemi)
    sons[7] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/hitenemy.ogg",0,0,0); //shot 1
    sons[8] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/killenemy.ogg",0,0,0); // shot 2
    sons[9] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/surprise.ogg",0,0,0); // surprise
    sons[10] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/monte.ogg",0,0,0); // monte
    sons[11] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/descend.ogg",0,0,0); // descend
    sons[12] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/chute.ogg",0,0,0); // chute
    sons[13] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/item.ogg",0,0,0); // item
    sons[14] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/rupee.ogg",0,0,0); // rubis
    sons[15] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/heart.ogg",0,0,0); // coeur
    sons[16] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/bomb.ogg",0,0,0); // bombe
    sons[17] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/textnext.ogg",0,0,0); // suite texte
    sons[18] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/textend.ogg",0,0,0); // fin texte
    sons[19] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/happy.ogg",0,0,0); // trouve objet
    sons[20] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/door.ogg",0,0,0); // ouvre porte
    sons[21] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/pics.ogg",0,0,0); // pics contre mur
    sons[22] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/sword.ogg",0,0,0); // pe
    sons[23] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/SwordCharging.ogg",0,0,0); // charge
    sons[24] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/Sword360.ogg",0,0,0); // spin
    sons[25] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/shoot.ogg",0,0,0); // flche
    sons[26] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/hookshot.ogg",0,0,0); // grappin
    sons[27] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/stamp.ogg",0,0,0); // pose bombe
    sons[28] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/magic.ogg",0,0,0); // magie
    sons[29] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/burn.ogg",0,0,0); // brle
    sons[30] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/hammer.ogg",0,0,0); // marteau
    sons[31] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/plouf.ogg",0,0,0); // plouf
    sons[32] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/danger.ogg",0,0,0); // danger
    sons[33] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/hurt.ogg",0,0,0); // link se blesse
    sons[34] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/porte.ogg",0,0,0); // porte objet
    sons[35] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/lance.ogg",0,0,0); // lance objet
    sons[36] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/casse.ogg",0,0,0); // casse objet
    sons[37] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/charge.ogg",0,0,0); // charge magie
    sons[38] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/buisson.ogg",0,0,0); // coupe buisson
    sons[39] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/pousse.ogg",0,0,0); // pousse caisse
    sons[40] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/envol.ogg",0,0,0); // chant envol
    sons[41] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/inverse.ogg",0,0,0); // invers
    sons[42] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/accelere.ogg",0,0,0); // acclr
    sons[43] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/splash.ogg",0,0,0); // ...

}

void Audio::freeSounds() {
    if (SOUND) {
        for (int i = 0; i < 44; i++) FSOUND_Sample_Free(sons[i]);
        delete[] sons;
    }
}

void Audio::playSound(int id, int chl) {
    if (SOUND) //Mix_PlayChannel(chl,sons[id],0);
        FSOUND_PlaySound(chl, sons[id]);
}

void Audio::stopSound() {
    if (SOUND) {FSOUND_StopSound(FSOUND_ALL); musiqueId=0;}
}

void Audio::playMusic(int id) {
    if (SOUND) {
        if (id == 2 || id == 5 || id == 7 || id == 8 || id == 9 || id == 14) id = 1;
        if (id == 20 || id == 22 || id == 24) id = 17;
        if (id == 29) id = 19;
        if (id == 32 || id == 34 || id == 35 || id == 39 || id == 44) id = 31;
        if (id >= 61 && id <= 78) id = 61;
        if (id == 82 || id == 84 || id == 94 || id == 98 || id == 99
        || id == 101 || id == 107 || id == 108 || id == 109 || id == 112
        || id == 117 || id == 118 || id == 122 || id == 128 || id == 129
        || id == 142 || id == 143 || id == 148 || id == 149) id = 81; //magasins
        if (id == 91 || id == 120 || id == 132) id = 85; //potions
        if (id == 102 || id == 136) id = 80; //bar
        if (id == 105 || id == 106 || id == 131 || id == 139 || id == 147) id = 95; //maire
        if (id == 152 || id == 153) id = 151; //temple temps
        if (musiqueId != id) {
            musiqueId = id;
            if (specialId == 0) {
                FMUSIC_StopSong(music);
                FMUSIC_FreeSong(music);
                music = choixMusique(id);
                if (previous_volume == -1) previous_volume = FMUSIC_GetMasterVolume(music);
                FMUSIC_SetMasterVolume(music, volume);
                FMUSIC_SetLooping(music, 1);
                FMUSIC_PlaySong(music);
                specialId = 0;
            }
        }
    }
}

bool Audio::isSpecial() {return (specialId>0);}

void Audio::stopMusic() {
    if (SOUND) FMUSIC_StopSong(music);
}

void Audio::replayMusic() {
    if (SOUND) FMUSIC_PlaySong(music);
}

FMUSIC_MODULE* Audio::choixMusique(int id) {
    switch (id) {
        case 1 : case 2 : case 7 : return FMUSIC_LoadSong("data/music/PlaineP.mid");
        case 3 : return FMUSIC_LoadSong("data/music/VilleCP.mid");
        case 4 : return FMUSIC_LoadSong("data/music/TerresS.mid");
        case 6 : return FMUSIC_LoadSong("data/music/BoisPerdus.mid");
        case 8 : return FMUSIC_LoadSong("data/music/CimetiereP.mid");
        case 10 : return FMUSIC_LoadSong("data/music/VillageMP.mid");
        case 11 : return FMUSIC_LoadSong("data/music/LacP.mid");
        case 12 : return FMUSIC_LoadSong("data/music/DesertP.mid");
        case 13 : return FMUSIC_LoadSong("data/music/Cocorico.mid");
        case 15 : return FMUSIC_LoadSong("data/music/MontP.mid");
        case 16 : return FMUSIC_LoadSong("data/music/Foret.mid");
        case 17 : return FMUSIC_LoadSong("data/music/Plaine.mid");
        case 18 : return FMUSIC_LoadSong("data/music/Cite.mid");
        case 19 : return FMUSIC_LoadSong("data/music/Chateau.mid");
        case 21 : return FMUSIC_LoadSong("data/music/Lanelle.mid");
        case 23 : return FMUSIC_LoadSong("data/music/Cimetiere.mid");
        case 25 : return FMUSIC_LoadSong("data/music/VillageM.mid");
        case 26 : return FMUSIC_LoadSong("data/music/Lac.mid");
        case 27 : return FMUSIC_LoadSong("data/music/Desert.mid");
        case 28 : return FMUSIC_LoadSong("data/music/VillageO.mid");
        case 30 : return FMUSIC_LoadSong("data/music/Mont.mid");
        case 31 : case 32 : case 34 : case 35 :
            return FMUSIC_LoadSong("data/music/Ombre.mid");
        case 33 : return FMUSIC_LoadSong("data/music/VilleF.mid");
        case 36 : return FMUSIC_LoadSong("data/music/BoisPerdusF.mid");
        case 37 : return FMUSIC_LoadSong("data/music/Cascades.mid");
        case 38 : return FMUSIC_LoadSong("data/music/CimetiereF.mid");
        case 40 : return FMUSIC_LoadSong("data/music/VillageMF.mid");
        case 41 : return FMUSIC_LoadSong("data/music/LacF.mid");
        case 42 : return FMUSIC_LoadSong("data/music/DesertF.mid");
        case 43 : return FMUSIC_LoadSong("data/music/VillageOF.mid");
        case 45 : return FMUSIC_LoadSong("data/music/MontF.mid");
        case 46 : return FMUSIC_LoadSong("data/music/Courage.mid");
        case 47 : return FMUSIC_LoadSong("data/music/Sagesse.mid");
        case 48 : return FMUSIC_LoadSong("data/music/Force.mid");
        case 49 : return FMUSIC_LoadSong("data/music/Abysses.mid");
        case 50 : return FMUSIC_LoadSong("data/music/PyramideF.mid");
        case 51 : return FMUSIC_LoadSong("data/music/PyramideP.mid");
        case 52 : return FMUSIC_LoadSong("data/music/Ordinn.mid");
        case 53 : return FMUSIC_LoadSong("data/music/Air.mid");
        case 54 : return FMUSIC_LoadSong("data/music/Glace.mid");
        case 55 : return FMUSIC_LoadSong("data/music/Feu.mid");
        case 56 : return FMUSIC_LoadSong("data/music/Titre.mid");
        case 57 : return FMUSIC_LoadSong("data/music/DFinal.mid");
        case 58 : return FMUSIC_LoadSong("data/music/Casino.mid");
        case 59 : return FMUSIC_LoadSong("data/music/Gemme.mid");
        case 60 : return FMUSIC_LoadSong("data/music/DestinationF.mid");
        case 61 : return FMUSIC_LoadSong("data/music/Cave.mid");
        case 79 : return FMUSIC_LoadSong("data/music/Home.mid");
        case 80 : return FMUSIC_LoadSong("data/music/Bar.mid");
        case 81 : return FMUSIC_LoadSong("data/music/Magasin.mid");
        case 83 : return FMUSIC_LoadSong("data/music/Maison.mid");
        case 85 : return FMUSIC_LoadSong("data/music/Potion.mid");
        case 89 : return FMUSIC_LoadSong("data/music/Jeu.mid");
        case 95 : return FMUSIC_LoadSong("data/music/Maire.mid");
        case 119 : return FMUSIC_LoadSong("data/music/Cafe.mid");
        case 144 : return FMUSIC_LoadSong("data/music/Sages.mid");
        case 150 : return FMUSIC_LoadSong("data/music/Opera.mid");
        case 151 : return FMUSIC_LoadSong("data/music/Epee.mid");
        case 154 : return FMUSIC_LoadSong("data/music/Prison.mid");
        case 155 : return FMUSIC_LoadSong("data/music/ChateauF.mid");
        case 218 : return FMUSIC_LoadSong("data/music/probleme.mid");
        case 219 : return FMUSIC_LoadSong("data/music/Epee.mid");
        case 180 : return FMUSIC_LoadSong("data/music/Titre.mid");
        case 190 : return FMUSIC_LoadSong("data/music/Selection.mid");
        case 199 : return FMUSIC_LoadSong("data/music/Nuit.mid");
        case 200 : return FMUSIC_LoadSong("data/music/Debut.mid");
        default : return FMUSIC_LoadSong("data/music/Maison.mid");
    }
}

void Audio::playSpecial(int id) {
    if (SOUND) {
        if (specialId != id) {
            FMUSIC_StopSong(music);
            FMUSIC_FreeSong(music);
            music = choixSpecial(id);
            if (previous_volume == -1) previous_volume = FMUSIC_GetMasterVolume(music);
            FMUSIC_SetMasterVolume(music, volume);
            FMUSIC_SetLooping(music, 1);
            FMUSIC_PlaySong(music);
            specialId=id;
        }
    }
}

void Audio::stopSpecial() {
    if (!specialId) return;
    int tmp = musiqueId;
    musiqueId = 0;
    specialId = 0;
    playMusic(tmp);
}

FMUSIC_MODULE* Audio::choixSpecial(int id) {
    switch (id) {
        case 1 : return FMUSIC_LoadSong("data/music/Boss.mid");
        case 2 : return FMUSIC_LoadSong("data/music/Mort.mid");
        case 3 : return FMUSIC_LoadSong("data/music/Epee.mid");
        case 4 : return FMUSIC_LoadSong("data/music/BossF.mid");
        case 5 : return FMUSIC_LoadSong("data/music/Fin.mid");
        case 6 : return FMUSIC_LoadSong("data/music/BossM.mid");
        case 7 : return FMUSIC_LoadSong("data/music/Area81.mid");
        case 8 : return FMUSIC_LoadSong("data/music/OniLink.mid");
        case 9 : return FMUSIC_LoadSong("data/music/probleme.mid");
        case 10 : return FMUSIC_LoadSong("data/music/Harpie.mid");
        case 11 : return FMUSIC_LoadSong("data/music/Crabe.mid");
        case 12 : return FMUSIC_LoadSong("data/music/Imp.mid");
        case 13 : return FMUSIC_LoadSong("data/music/Masamune.mid");
        case 14 : return FMUSIC_LoadSong("data/music/ZoraS.mid");
        case 15 : return FMUSIC_LoadSong("data/music/Marlag.mid");
        case 16 : return FMUSIC_LoadSong("data/music/Fantomas.mid");
        case 17 : return FMUSIC_LoadSong("data/music/Vampire.mid");
        case 18 : return FMUSIC_LoadSong("data/music/Araignee.mid");
        case 19 : return FMUSIC_LoadSong("data/music/Plumes.mid");
        case 20 : return FMUSIC_LoadSong("data/music/Garuda.mid");
        case 21 : return FMUSIC_LoadSong("data/music/Heckran.mid");
        case 22 : return FMUSIC_LoadSong("data/music/Sun.mid");
        case 23 : return FMUSIC_LoadSong("data/music/Orcus.mid");
        case 24 : return FMUSIC_LoadSong("data/music/Agahnim.mid");
        case 25 : return FMUSIC_LoadSong("data/music/Zelda.mid");
        case 26 : return FMUSIC_LoadSong("data/music/AgahnimFinal.mid");
        case 27 : return FMUSIC_LoadSong("data/music/GanondorfFinal.mid");
        case 28 : return FMUSIC_LoadSong("data/music/Quizz.mid");
        case 29 : return FMUSIC_LoadSong("data/music/FinalBattle.mid");
        default : return FMUSIC_LoadSong("data/music/Boss.mid");
    }
}
